(function(window, $, Slim, undefined){
    
    // Namespace
    var PA = {};
    
    // ID selectors
    PA.idSelectors = {
        addresseeInput: '#addressee',
        an: '#an',
        bios: '#bios',
        bioNameInput: '#name',
        content: '#content',
        dateFrom: '#dateFrom', 
        dateTo: '#dateTo', 
        docs: '#docs', 
        nuau: '#nuau',
        nuen: '#nuen',
        organInput: '#organ',
        placeInput: '#place', 
        subs: '#subs',
        subTitleInput: '#title',
        senderInput: '#sender',
        verfasserInput: '#verfasser',
        we: '#we'
    };
    
    // Class selecors
    PA.selectors = {
        addresseeSuggestion: '.addressee-suggestion',
        additionalOptions: '.additional-options', 
        bioContainer: '.bio-container',
        bioLink: '.bio-link',
        bioLinkIcon: '.bio-link-icon',
        bioLinkItem: '.bio-link-item',
        bioNameSuggestion: '.bio-name-suggestion',
        breadcrumb: '.breadcrumb',
        closeBtn : '.close-btn',
        customRadio: '.custom-radio',
        docBodyContainer: '.doc-body-container',
        docContainer: '.doc-container',
        docSchreiber: '.doc-schreiber',
        docLinkItems: '.doc-link-items',
        docLinkItemsGroup: '.doc-link-items-group',
        docLinkItem: '.doc-link-item',
        docSubject: '.doc-subject',
        docTypeControlBtn: '.doc-type-control-btn',
        gndBtn: '.gnd-btn',
        linkItems: '.link-items',
        linkItem: '.link-item',
        magicLink: '.magic-link',
        accordion: '.accordion',
        accoSection: '.acco-section',
        moveToTopBtn: '.move-to-top-btn',
        nmbResults: '.nmb-results',
        organSuggestion: '.organ-suggestion',
        pageInfo: '.page-info',
        placeSuggestion: '.place-suggestion',
        pdfLink: '.pdf-link',
        resource: '.resource',
        resultsContainer: '.results-container',
        resourceLinkIcon: '.resource-link-icon',
        resourceLink: '.resource-link',
        schreiberName: '.schreiber-name',
        senderSuggestion: '.sender-suggestion',
        singleViewBtn: '.single-view-btn',
        stickyHeader: '.slim-sticky-header',
        subContainer: '.sub-container',
        subLink: '.sub-link',
        subLinkIcon: '.sub-link-icon',
        subLinkItem: '.sub-link-item',
        subTitleSuggestion: '.sub-title-suggestion',
        accoSectionTitle: '.acco-section-title',
        verfasserSuggestion: '.verfasser-suggestion'
    };

    // Classnames
    PA.classNames = {
        active: 'active',
        bioContainer: 'bio-container',
        bioLinkItem: 'bio-link-item',
        closeBtn: 'close-btn',
        copied: 'copied',
        docContainer: 'doc-container',
        docLinkItem: 'doc-link-item',
        disabled: 'disabled',
        faCaretLeft: 'fas fa-arrow-left',
        faCaretUp: 'fas fa-arrow-up',
        faChevronDown: 'fas fa-chevron-circle-down',
        faChevronUp: 'fas fa-chevron-circle-up',
        faClose: 'fa fa-window-close',
        faExtLink: 'fas fa-external-link-alt',
        faMinus: 'fas fa-minus-circle',
        faPlus: 'fas fa-plus-circle',
        faTimes: 'fas fa-times',
        linkItemAjax: 'link-item-ajax',
        moveToTopBtn: 'move-to-top-btn',
        moveLeftBtn: 'move-left-btn',
        newTabBtn: 'new-tab-btn',
        singleViewBtn: 'single-view-btn',
        slimBackToTopBtnRight: 'slim-back-to-top-btn-right',
        subContainer: 'sub-container',
        subLinkItem: 'sub-link-item',
        toggleResourceBtn: 'toggle-resource-btn'
    };
    
    // JS options for the Pacelli website 
    PA.options = {}; 
    
    // Options for ajax calls
    PA.options.ajax = {}; 
    
    // Urls for ajax calls
    PA.options.ajax.urls = {
        attachInDoc: './resources/html/ajax/get-linked-attachments.html',
        getAddressees: './resources/html/ajax/get-addressees.html',
        getBioNames: './resources/html/ajax/get-bio-names.html',
        getDocBody: './resources/html/ajax/get-doc-body.html',
        getOrgans: './resources/html/ajax/get-organs.html',
        getPlaces: './resources/html/ajax/get-places.html',
        getSenders: './resources/html/ajax/get-senders.html',
        getSubTitles: './resources/html/ajax/get-sub-titles.html',
        getVerfassers: './resources/html/ajax/get-verfassers.html',
        searchBios: './resources/html/ajax/get-bios-by-name.html',
        searchSubs: './resources/html/ajax/get-subs-by-title.html'
    };     

    // Options for (Slim.)tooltips
    PA.options.tooltips = {
        basic: {
            className: 'small-tooltip dark-tooltip'
        },
        autocomplete: {
            className: 'shadow-tooltip scroll-tooltip'
        }
    };
    
    // Options for (Slim.)popups
    PA.options.popups = {
        basic: {}
    },
    
    PA.options.datepicker = {
        lang: 'de',
        yearStart: '1917',
        yearEnd: '1929'
    }
    
    // Pages that have to be loaded completely before initialisation
    PA.options.loadFirstPages = [];

   
    /* ------------------------------------------------------------------------------------------------------------------------------------------------
     * Mixed utility functions
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
  
    PA.mixed = (function(){
        
     // Create a button to toggle additional search options
        function addToggleOptionsBtn(formSelector, optionsSelector){
            
            var $btn = $(document.createElement('div'));
            var $options = $(optionsSelector);
            
            if (location.pathname.includes("/en/")) {
                var translFurtherOptions = 'Further search options';
            } else {
                 var translFurtherOptions = 'Weitere Suchoptionen';
            }
               
            function toggleOptions(e){ e.stopPropagation();
                 
                if ($btn.hasClass('open')){
                
                    $options.hide();
                    $btn.find('i').attr('class', 'fas fa-chevron-down'); 
                    $btn.removeClass('open');
                    
                } else {
                
                    $options.show();
                    $btn.find('i').attr('class', 'fas fa-chevron-up');  
                    $btn.addClass('open');
                }
            };
            
            $btn.addClass('additional-options-btn open')
                .text(translFurtherOptions)
                .append('<i class="fas fa-chevron-down"/>')
                .insertBefore($options)
                .click(toggleOptions)
                .click();
        };   
    
        // Scroll to a selected DOM node
        function scrollToElem(selector, marginTop){
        
            if ($(selector).length === 0){ return false; }
            
            $('html, body').animate({
                scrollTop: ($(selector).offset().top + marginTop || 0)
            }, 500);
        }; 
        
        // Helper for initAutocompleteInput
        function doAfterRequest(fn, fnArgs) {
        
            if ($.active > 0) {
                window.setTimeout(doAfterRequest, 1000, fn, fnArgs);
            } else {
                fn.apply(null, fnArgs);
            }
        };
        
        // Helper for initAutocompleteInput
        function initSuggestions(suggestionsSelector, $input){ 
        
            $(suggestionsSelector).click(function(){
                $input.val($(this).text());
                $(document).click();
            }) 
        };
        
        // Insert the results of a autocomplete input (fetched via ajax) into the input itself
        function initAutocompleteInput(inputSelector, suggestionsSelector){
        
            $(inputSelector).on('input click', function(){
                window.setTimeout(doAfterRequest, 1000, initSuggestions, [suggestionsSelector, $(this)]);
            }); 
        };
       
        // Get all parameters contained in the current page url as hash
        function getUrlParams(){
        
            var params = {};
            var parts = window.location.href.replace(/[?&]+([^=&]+)=([^&]*)/gi, function(m,key,value) {
                params[key] = value;
            });
            
            return params;
        };
        
        // Copy text to clipboard
        function copyToClipboard($this){
        
            $this.select(); 
            document.execCommand('copy');
            setTimeout(function(){ $this.blur();}, 2000 );
        }; 
        
        // Check a set of checkboxes
        function checkCheckboxes(ids){
        
            ids.forEach(function(id){
                $(id).prop('checked', true); 
            });
        }; 
        
        // Make sure that the page height is not smaller than the window height
        function adaptPageHeight(){
            
            var cHeight = $('body').height(); 
            var wHeight = $(window).height();
            var diff = Math.round(wHeight - cHeight); 
    
            if (diff > 0){     
            
                var mTopRaw = $('footer').css('margin-top'); 
                var mTop = parseInt(mTopRaw.replace('px', ''));
                
                $('footer').css('margin-top', (diff + mTop) + 'px');
            } 
        };
        
        // Hide placeholders when clicking an input
        function adaptInputPlaceholders(){
        
            $('input[placeholder],textarea[placeholder]').focus(function(){
               $(this).data('placeholder', $(this).attr('placeholder'))
                      .attr('placeholder', '');
            }).blur(function(){
               $(this).attr('placeholder', $(this).data('placeholder'));
            });  
        };
           
        return {
            addToggleOptionsBtn: addToggleOptionsBtn,
            scrollToElem: scrollToElem,
            initAutocompleteInput: initAutocompleteInput,
            getUrlParams: getUrlParams,
            copyToClipboard: copyToClipboard,
            checkCheckboxes: checkCheckboxes,
            adaptPageHeight: adaptPageHeight,
            adaptInputPlaceholders: adaptInputPlaceholders
        }
    }());


    /* ------------------------------------------------------------------------------------------------------------------------------------------------
     * Localizations for the Date Picker Widget
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */    

        
    PA.datePickerLocalizations = (function() {

        var getOptions = function(){

            const options = { yearStart: '1917', yearEnd: '1929'};
        
            if (location.pathname.includes("/en/"))  options.lang = 'en'
            else options.lang = 'de';
        
            return options
        }
        
         var dateFrom = function() {
           
            if (location.pathname.includes("/en/")) return '1917-04-20'
            else return '20.04.1917';
        };
        
        var dateTo = function() {
            if (location.pathname.includes("/en/")) return '1929-12-09'
            else return '09.12.1929';
        };
        
        var dateFromTitle = function() {
            if (location.pathname.includes("/en/")) return 'Time frame from'
            else return 'Zeitraum ab dem';
        };
        
        var dateToTitle = function() {
            
            if (location.pathname.includes("/en/")) return 'Time frame to'
            else return 'Zeitraum bis zum';
        };
        
        return {
            getOptions: getOptions,
            dateFrom: dateFrom,
            dateTo: dateTo,
            dateFromTitle: dateFromTitle,
            dateToTitle: dateToTitle};
    
    })();

    
    /* ------------------------------------------------------------------------------------------------------------------------------------------------
     * Functionality for the single pages of the Pacelli website
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
    
    // Functionality for the single pages of the Pacelli website
    PA.pages = {}; 
    
    /* Bios
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
    
    // kurzbiografien.html
    PA.pages.bios = function(){

        // Create an autocomplete input that provides link-items to single bios
        Slim.createAutocomplteInput(PA.idSelectors.bioNameInput, {
            ajaxUrl:PA.options.ajax.urls.searchBios, 
            container: PA.options.tooltips.autocomplete
        });  
        
        // Init the link items returned by the current request so that they can toggle the associated bio
        new PAO.ResultSet(PA.selectors.resultsContainer, 'bio'); 
    };
    
    // kurzbiografie.html
    PA.pages.bio = function(){
    
        var $title = $(PA.selectors.accoSectionTitle);
        var $bio = $(PA.selectors.resource); 
        
        new PAO.Bio($bio); 
        new PAO.AccoTitle($title, $title.attr('data-type'), $title.attr('data-idno')); 
        
        $(PA.selectors.magicLink).on('click', function(){ PA.mixed.copyToClipboard($(this));} );
    };
    
    // suche-kurzbiografien.html
    PA.pages.searchBios = function(){
        
        // Create an autocomplete input that provides bio name suggestions
        Slim.createAutocomplteInput(PA.idSelectors.bioNameInput, {
            ajaxUrl:PA.options.ajax.urls.getBioNames, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.bioNameInput, PA.selectors.bioNameSuggestion); 
        
        // Init the link items returned by the current request so that they can toggle the associated bio
        new PAO.ResultSet(PA.selectors.resultsContainer, 'bio'); 
    };
    
    /* Subs
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
    
    // schlagwoerter.html
    PA.pages.subs = function(){

        // Create an autocomplete input that provides link-items to single subs
        Slim.createAutocomplteInput(PA.idSelectors.subTitleInput, {
            ajaxUrl:PA.options.ajax.urls.searchSubs, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        // Init the link items returned by the current request so that they can toggle the associated sub
        new PAO.ResultSet(PA.selectors.resultsContainer, 'sub'); 
    };
    
    // schlagwort.html
    PA.pages.sub = function(){
    
        var $title = $(PA.selectors.accoSectionTitle);
        var $sub = $(PA.selectors.resource); 
        
        new PAO.Bio($sub); 
        new PAO.AccoTitle($title, $title.attr('data-type'), $title.attr('data-idno')); 
        
        $(PA.selectors.magicLink).on('click', function(){PA.mixed.copyToClipboard($(this))});
    };
    
    // suche-schlagwoerter.html
    PA.pages.searchSubs = function(){
        
        // Create an autocomplete input that provides sub title suggestions
        Slim.createAutocomplteInput(PA.idSelectors.subTitleInput, {
            ajaxUrl:PA.options.ajax.urls.getSubTitles, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.subTitleInput, PA.selectors.subTitleSuggestion); 
        
        // Init the link items returned by the current request so that they can toggle the associated bio
        new PAO.ResultSet(PA.selectors.resultsContainer, 'sub'); 
    };
    
    /* Docs
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
   
    // dokumente-nach-archiven.html
    PA.pages.docsArchives = function(){
        
        new PAO.ResultSet(PA.selectors.resultsContainer, 'doc');
    };
   
   
    // dokumente-nach-themen.html
    PA.pages.docsTopics = function(){
        
        new PAO.ResultSet(PA.selectors.resultsContainer, 'doc');
    };
    
    // dokumente-chronologisch.html
    PA.pages.docsChrono = function(){
        
        new PAO.ResultSet(PA.selectors.resultsContainer, 'doc');
    };
    
    // dokument.html
    PA.pages.doc = function(){
        
        var referrer = document.referrer;   
        var $pseudoBcrs = $(PA.selectors.breadcrumb);
        var $lastPseudoBcr = $pseudoBcrs.find('li').last(); 
        var $docContainer = $(PA.selectors.docContainer); 
        
        function updateDocBody(e){
        
            e.preventDefault(); 
            
            var $btn = $(this);
            var href = $btn.attr('href');
            var $docContainer = $btn.closest(PA.selectors.docContainer);
            var data = {
                idno: $docContainer.attr('data-idno'),
                ignore: href.substring(href.lastIndexOf('=')+1)        
            }
            var loadingSpinner = Slim.LoadingSpinner().append();
            
            $.ajax({
                url: PA.options.ajax.urls.getDocBody,
                data: data
            }).done(function(data){
            
                // Toggle 'active' in sticky header schreibers and normal schreibers
                $(PA.selectors.schreiberName).removeClass('active');
                $(PA.selectors.docSchreiber).find(PA.selectors.schreiberName + ':eq(' + ($btn.index()-1) + ')').addClass('active');
            
                // Remove old docBody + subject
                $($(PA.selectors.docSubject)).remove(); 
                $($(PA.selectors.docBodyContainer)).remove();
                
                // Append new docBody + subject
                $(PA.selectors.docSchreiber).not($(PA.selectors.stickyHeader)).after(data); 
                
         
                $btn.closest($docContainer).data('sticky-header').adaptToContainer($docContainer, {top:450})
                
            }).always(function(){
                loadingSpinner.remove();
            });  
        };
        
        /* Replace the "pseudo" breacrumbs with the amended breadcrumbs of the previous page due to the fact 
         * that breadcrumbs - unless on all other pages - can't e generated from url params */
        jQuery.get(referrer, function(data){
            
            var $bcrs = $(data).find(PA.selectors.breadcrumb);
            var $lastBcr = $bcrs.find('li').last(); 
            var $lastBcrLink = $(document.createElement('a'))
                .attr('href', referrer)
                .text($lastBcr.text());
            
            // Change last breadcrumb item to a link item
            $lastBcr.removeClass('active')
                .empty()
                .append($lastBcrLink); 
             
            // Add new last breadcrumb item and append the amened breadcrumbs
            $bcrs.append($lastPseudoBcr)
                .insertAfter($pseudoBcrs);   
                
            $pseudoBcrs.remove(); 
        });
        
        new PAO.Doc(PA.selectors.docContainer);
        
        $(PA.selectors.accoSectionTitle).each(function(){
            new PAO.AccoTitle(this, $(this).attr('data-type'), $(this).attr('data-idno'), {moveToTopIcon: true});
        });
        
        // Layermodel 
        if ($(PA.selectors.docSchreiber).length > 0){
        
            var stickyHeader = new Slim.StickyHeader(PA.selectors.docSchreiber).appendTo($docContainer).adaptToContainer($docContainer, {top:450});
        
            $(PA.selectors.schreiberName + ':last-child').addClass('active'); 
            $(PA.selectors.schreiberName).on('click', updateDocBody);
            
            $docContainer.data('sticky-header', stickyHeader);
        } 
        
        $(PA.selectors.magicLink).on('click', function(){PA.mixed.copyToClipboard($(this))}); 
    }
    
    /* suche-dokumente.html */
    PA.pages.searchDocs = function(){

        // Check all checkboxed if there is currently no request
        if (location.search === ''){

            PA.mixed.checkCheckboxes([
                PA.idSelectors.nuau, 
                PA.idSelectors.nuen, 
                PA.idSelectors.we,
                PA.idSelectors.an
            ]);
        } 
        
        PA.mixed.addToggleOptionsBtn('form', PA.selectors.additionalOptions);
       
        // Scroll results to top
        PA.mixed.scrollToElem(PA.selectors.resultsContainer, -32);
        
        // Autocomplete input "Person"
        Slim.createAutocomplteInput(PA.idSelectors.bioNameInput, {
            ajaxUrl:PA.options.ajax.urls.getBioNames, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.bioNameInput, PA.selectors.bioNameSuggestion);
        
        // Autocomplete input "Schlagwort"
        Slim.createAutocomplteInput(PA.idSelectors.subTitleInput, {
            ajaxUrl:PA.options.ajax.urls.getSubTitles, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.subTitleInput, PA.selectors.subTitleSuggestion); 
        
        // Autocomplete input "Absender"
        Slim.createAutocomplteInput(PA.idSelectors.senderInput, {
            ajaxUrl:PA.options.ajax.urls.getSenders, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.senderInput, PA.selectors.senderSuggestion);
        
        // Autocomplete input "Empfänger"
        Slim.createAutocomplteInput(PA.idSelectors.addresseeInput, {
            ajaxUrl:PA.options.ajax.urls.getAddressees, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.addresseeInput, PA.selectors.addresseeSuggestion);
        
        // Autocomplete input "Verfasser"
        Slim.createAutocomplteInput(PA.idSelectors.verfasserInput, {
            ajaxUrl:PA.options.ajax.urls.getVerfassers, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.verfasserInput, PA.selectors.verfasserSuggestion);
        
        // Autocomplete input "Zeitschrift/Zeitschrift"
        Slim.createAutocomplteInput(PA.idSelectors.organInput, {
            ajaxUrl:PA.options.ajax.urls.getOrgans, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.organInput, PA.selectors.organSuggestion);
        
        // Autocomplete input "Ort"
        Slim.createAutocomplteInput(PA.idSelectors.placeInput, {
            ajaxUrl:PA.options.ajax.urls.getPlaces, 
            container: PA.options.tooltips.autocomplete
        }); 
        
        PA.mixed.initAutocompleteInput(PA.idSelectors.placeInput, PA.selectors.placeSuggestion); 
        
        // Init datepickers
        Slim.bindDatepickerTo(PA.idSelectors.dateFrom, PA.datePickerLocalizations.dateFromTitle(), PA.datePickerLocalizations.getOptions(), PA.datePickerLocalizations.dateFrom() );
        Slim.bindDatepickerTo(PA.idSelectors.dateTo, PA.datePickerLocalizations.dateToTitle(),  PA.datePickerLocalizations.getOptions(), PA.datePickerLocalizations.dateTo() );
        
        // Init results
        new PAO.ResultSet(PA.selectors.resultsContainer, 'doc');
    };
     
    
    /* Suche im Gesamtbestand
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
    
    /* suche-gesamtbestand.html */
    PA.pages.searchAll = function(){

        // Check all checkboxed if there is currently no request
        if (location.search === ''){

            PA.mixed.checkCheckboxes([
                PA.idSelectors.bios, 
                PA.idSelectors.docs, 
                PA.idSelectors.subs
            ]);
            
        };
        
        // Init results
        new PAO.ResultSet(PA.selectors.resultsContainer, 'mixed');
    }; 
    
    
    /* ------------------------------------------------------------------------------------------------------------------------------------------------
     * Init page
     * ------------------------------------------------------------------------------------------------------------------------------------------------ */
     
    // Initialize a single page of the Pacelli website
    function init(){
        
        var pageId = $(PA.selectors.pageInfo).attr('data-pageid');
        var pageFn = PA.pages[pageId]; 
        
        // On every page
        Slim.BackToTopBtn({className: PA.classNames.slimBackToTopBtnRight}).append();
        
        PA.mixed.adaptInputPlaceholders();
        
        // Specific functions for single pages
        if (pageFn){ pageFn(); }
    };

    // Invoke init function
    // Pages stated in 'PA.options.loadFirstPages' have to be loaded completely before initialisation
    if (PA.options.loadFirstPages.indexOf(window.location.pathname.substring(1)) > -1){
    
        $(window).on('load', init);
        
    } else {
    
        $(init); 
    }
    
}(window, $, Slim, undefined));